Package: ddalpha
Type: Package
Title: Depth-Based Classification and Calculation of Data Depth
Version: 1.3.16
Date: 2024-09-29
Authors@R: c(person("Oleksii", "Pokotylo", role=c("aut", "cre"),
                                          email = "alexey.pokotylo@gmail.com"),
             person("Pavlo", "Mozharovskyi", role=c("aut"),
                                          email = "pavlo.mozharovskyi@telecom-paris.fr"),
             person("Rainer", "Dyckerhoff", role=c("aut"),
                                          email = "rainer.dyckerhoff@statistik.uni-koeln.de"),
             person("Stanislav", "Nagy", role=c("aut"),
                                          email = "nagy@karlin.mff.cuni.cz"))
Depends: R (>= 2.10), stats, utils, graphics, grDevices, MASS, class,
        robustbase, sfsmisc, geometry
Imports: Rcpp (>= 0.11.0)
LinkingTo: BH, Rcpp
Description: Contains procedures for depth-based supervised learning, which are entirely non-parametric, in particular the DDalpha-procedure (Lange, Mosler and Mozharovskyi, 2014 <doi:10.1007/s00362-012-0488-4>). The training data sample is transformed by a statistical depth function to a compact low-dimensional space, where the final classification is done. It also offers an extension to functional data and routines for calculating certain notions of statistical depth functions. 50 multivariate and 5 functional classification problems are included. (Pokotylo, Mozharovskyi and Dyckerhoff, 2019 <doi:10.18637/jss.v091.i05>).
License: GPL-2
NeedsCompilation: yes
Packaged: 2024-09-29 13:54:41 UTC; alexis
Author: Oleksii Pokotylo [aut, cre],
  Pavlo Mozharovskyi [aut],
  Rainer Dyckerhoff [aut],
  Stanislav Nagy [aut]
Maintainer: Oleksii Pokotylo <alexey.pokotylo@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-30 22:50:02 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-24 18:03:04 UTC; unix
Archs: ddalpha.so.dSYM
